<?php
/*
Plugin Name: Connections
Plugin URI: http://connections-pro.com/
Description: A business directory and address book manager.
Version: 0.7.3.2
Author: Steven A. Zahm
Author URI: http://connections-pro.com/
Text Domain: connections
Domain Path: /lang

	Copyright 2009  Steven A. Zahm  (email : helpdesk@connections-pro.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2, as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/*
 * Credits:
 *
 * Connections is based on Little Black Book  1.1.2 by Gerald S. Fuller
 * Little Black Book is based on Addressbook 0.7 by Sam Wilson
 *
 * Uses a function here and there from NextGEN Gallery by Alex Rabe.
 * http://wordpress.org/extend/plugins/nextgen-gallery/
 *
 * Update Notice in plugin admin inspired by Changelogger 1.2.8 by Oliver Schlöbe.
 * http://wordpress.org/extend/plugins/changelogger/
 *
 * class.upload.php by Colin Verot.
 * http://www.verot.net
 *
 * TimThumb by Ben Gillbanks and Mark Maunder.
 * Based on work done by Tim McDaniels and Darren Hoyt.
 * http://code.google.com/p/timthumb/
 *
 * vCard class is a modified version by Troy Wolf.
 * http://www.troywolf.com/articles/php/class_vcard/
 *
 * Screen Options class by Janis Elsts
 * http://w-shadow.com/blog/2010/06/29/adding-stuff-to-wordpress-screen-options/
 *
 * spin.js by Felix Gnass
 * http://fgnass.github.com/spin.js/
 *
 * $.goMap() jQuery Google Maps Plugin by Jevgenijs Shtrauss
 * http://www.pittss.lv/jquery/gomap/
 *
 * MarkerClusterer jQuery Google Maps Marker Clustering Plugin by Xiaoxi Wu
 * http://gmaps-utility-library-dev.googlecode.com/svn/tags/markerclusterer/
 *
 * jQuery Validation Plugin by Jörn Zaefferer
 * http://bassistance.de/jquery-plugins/jquery-plugin-validation/
 *
 * Preloading GIF generated by Preloaders.net
 * http://www.preloaders.net/
 *
 * Email and URL validation methods by Gizmo Digital Fusion
 * http://wpcodesnippets.info/blog/two-useful-php-validation-functions.html
 *
 * Social media icons by WPZOOM.com
 * http://www.wpzoom.com/wpzoom/500-free-icons-wpzoom-social-networking-icon-set/
 * License CC BY-SA 3.0  http://creativecommons.org/licenses/by-sa/3.0/
 *
 * iTunes icon by Paul Robert Lloyd
 * http://www.iconfinder.com/icondetails/43155/48/itunes_icon
 * http://paulrobertlloyd.com/
 * License Attribution-ShareAlike 2.0 UK: England & Wales (CC BY-SA 2.0)
 *
 * Podcast icon by schollidesign
 * http://findicons.com/icon/94188/podcast_new)
 * License GNU/GPL
 *
 * vCard Icon by Oliver Twardowski
 * http://www.iconfinder.com/icondetails/50717/48/add_vcard_icon
 * License Free for commercial use
 * http://www.iconfinder.com/browse/iconset/iconset-addictive-flavour/#readme
 *
 * jQuery UI CSS and icons by helenhousandi
 * https://github.com/helenhousandi/wp-admin-jquery-ui
 */

/**
 *
 *
 * @TODO: Add support for SSL using the CN_PLUGIN_URL constant throughout.
 */


if ( ! class_exists( 'connectionsLoad' ) ) {

	class connectionsLoad {

		public $currentUser;
		public $options;
		public $retrieve;
		public $term;

		/**
		 * Stores the page hook values returned from the add_menu_page & add_submenu_page functions
		 *
		 * @var object
		 */
		public $pageHook;

		/**
		 * The Connections Settings API Wrapper class.
		 *
		 * @var object
		 */
		public $settings;

		public $message;

		/**
		 * Do the database upgrade.
		 *
		 * @var bool
		 */
		private $dbUpgrade;

		public function __construct() {

			$this->loadConstants();
			$this->loadDependencies();
			$this->initDependencies();

			// Init the options if there is a version change just in case there were any changes.
			if ( version_compare( $this->options->getVersion() , CN_CURRENT_VERSION ) < 0 ) $this->initOptions();

			// Activation/Deactivation hooks
			register_activation_hook( dirname( __FILE__ ) . '/connections.php', array( $this, 'activate' ) );
			register_deactivation_hook( dirname( __FILE__ ) . '/connections.php', array( $this, 'deactivate' ) );

			//@TODO: Create uninstall method to remove options and tables.
			// register_uninstall_hook( dirname(__FILE__) . '/connections.php', array('connectionsLoad', 'uninstall') );

			// Start this plug-in once all other plugins are fully loaded
			add_action( 'plugins_loaded', array( $this, 'start' ) );
		}

		public function start() {

			//$this->options->setDBVersion('0.1.8'); $this->options->saveOptions();

			// Load the translation files.
			load_plugin_textdomain( 'connections' , false , CN_DIR_NAME . '/lang' );

			/*
			 * Register the settings tabs shown on the Settings admin page tabs, sections and fields.
			 */
			add_filter( 'cn_register_settings_tabs' , array( 'cnRegisterSettings', 'registerSettingsTabs' ) , 10 , 1 );
			add_filter( 'cn_register_settings_sections' , array( 'cnRegisterSettings', 'registerSettingsSections' ) , 10 , 1 );
			add_filter( 'cn_register_settings_fields' , array( 'cnRegisterSettings', 'registerSettingsFields' ) , 10 , 1 );

			// Setup the current user object
			$current_user = wp_get_current_user();
			$this->currentUser->setID( $current_user->ID );

			// Register the JavaScript libraries.
			add_action( 'init', array( __CLASS__, 'registerScripts' ) );

			// Register the CSS.
			add_action( 'init', array( __CLASS__, 'registerCSS' ) );

			if ( is_admin() ) {

				// Store the PHP mememory limit
				$this->phpMemoryLimit = ini_get( 'memory_limit' );

				// Calls the method to load the admin menus.
				add_action( 'admin_menu', array( $this , 'adminMenu' ) );

				add_action( 'admin_init', array( $this, 'adminInit' ) );

				// Parse admin queries.
				add_action( 'admin_init', array( $this, 'adminActions' ) );

				/*
				 * Add the filter to update the user settings when the 'Apply" button is clicked.
				 * NOTE: This relies on the the Screen Options class by Janis Elsts
				 * NOTE: This filter must be init here otherwise it registers to late to be run.
				 */
				add_filter( 'set-screen-option', array( $this, 'managePageLimitSave' ), 10 , 3 );

			} else {

				// Enqueue the frontend scripts and CSS.
				add_action( 'wp_enqueue_scripts', array( __CLASS__, 'enqueueScripts' ) );
				add_action( 'wp_enqueue_scripts', array( __CLASS__, 'enqueueStyles' ) );

				// Parse front end queries.
				// add_action( 'parse_request', array( $this, 'userActions') );

				/*
				 * Process front end actions.
				 */
				add_action( 'template_redirect' , array( $this, 'frontendActions' ) );
			}

		}

		private function loadConstants() {

			global $wpdb, $blog_id;

			define( 'CN_LOG', FALSE );

			define( 'CN_CURRENT_VERSION', '0.7.3.2' );
			define( 'CN_DB_VERSION', '0.1.9' );

			/*
			 * By default Connections will install and run as a single site install
			 * on a multisite install. This requires manual activation on each sub-site.
			 *
			 * To run Connections in multisite mode.
			 * Add to wp-config.php: define('CN_MULTISITE_ENABLED', TRUE);
			 *
			 * @credit lancelot-du-lac
			 * @url http://wordpress.org/support/topic/plugin-connections-support-multisite-in-single-mode
			 */
			if ( ! defined( 'CN_MULTISITE_ENABLED' ) ) define( 'CN_MULTISITE_ENABLED', FALSE );


			/*
			 * Enable support for multi-site file locations.
			 * @url http://codex.wordpress.org/Determining_Plugin_and_Content_Directories#Available_Functions
			 */
			if ( is_multisite() && CN_MULTISITE_ENABLED ) {
				define( 'CN_IMAGE_PATH', WP_CONTENT_DIR . '/blogs.dir/' . $blog_id . '/connection_images/' );
				define( 'CN_IMAGE_BASE_URL', network_site_url( '/wp-content/blogs.dir/' . $blog_id . '/connection_images/' ) );
				define( 'CN_CUSTOM_TEMPLATE_PATH', WP_CONTENT_DIR . '/blogs.dir/' . $blog_id . '/connections_templates/' );
				define( 'CN_CUSTOM_TEMPLATE_URL', network_site_url( '/wp-content/blogs.dir/' . $blog_id . '/connections_templates/' ) );
			} else {
				define( 'CN_IMAGE_PATH', WP_CONTENT_DIR . '/connection_images/' );
				define( 'CN_IMAGE_BASE_URL', content_url() . '/connection_images/' );
				define( 'CN_CUSTOM_TEMPLATE_PATH', WP_CONTENT_DIR . '/connections_templates/' );
				define( 'CN_CUSTOM_TEMPLATE_URL', content_url() . '/connections_templates/' );
			}

			/*
			 * Set the table prefix accordingly depedning if Connections is installed on a multisite WP installation.
			 */
			$prefix = ( is_multisite() && CN_MULTISITE_ENABLED ) ? $wpdb->base_prefix : $wpdb->prefix;

			define( 'CN_ENTRY_TABLE', $prefix . 'connections' );
			define( 'CN_ENTRY_ADDRESS_TABLE', $prefix . 'connections_address' );
			define( 'CN_ENTRY_PHONE_TABLE', $prefix . 'connections_phone' );
			define( 'CN_ENTRY_EMAIL_TABLE', $prefix . 'connections_email' );
			define( 'CN_ENTRY_MESSENGER_TABLE', $prefix . 'connections_messenger' );
			define( 'CN_ENTRY_SOCIAL_TABLE', $prefix . 'connections_social' );
			define( 'CN_ENTRY_LINK_TABLE', $prefix . 'connections_link' );
			define( 'CN_ENTRY_DATE_TABLE', $prefix . 'connections_date' );

			define( 'CN_ENTRY_TABLE_META', $prefix . 'connections_meta' );
			define( 'CN_TERMS_TABLE', $prefix . 'connections_terms' );
			define( 'CN_TERM_TAXONOMY_TABLE', $prefix . 'connections_term_taxonomy' );
			define( 'CN_TERM_RELATIONSHIP_TABLE', $prefix . 'connections_term_relationships' );

			define( 'CN_DIR_NAME', plugin_basename( dirname( __FILE__ ) ) );
			define( 'CN_BASE_NAME', plugin_basename( __FILE__ ) );
			define( 'CN_PATH', plugin_dir_path( __FILE__ ) );
			define( 'CN_URL', plugin_dir_url( __FILE__ ) );

			define( 'CN_TEMPLATE_PATH', CN_PATH . 'templates/' );
			define( 'CN_TEMPLATE_URL', CN_URL . 'templates/' );
			define( 'CN_CACHE_PATH', CN_PATH . 'cache/' );
		}

		private function loadDependencies() {
			/**
			 *
			 *
			 * @TODO: Load dependencies as needed. For example load only classes
			 * needed in the admin and frontend
			 */
			//Current User objects
			require_once CN_PATH . '/includes/class.user.php'; // Required for activation
			//Terms Objects
			require_once CN_PATH . '/includes/class.terms.php'; // Required for activation
			//Category Objects
			require_once CN_PATH . '/includes/class.category.php'; // Required for activation, entry list
			//Retrieve objects from the db.
			require_once CN_PATH . '/includes/class.retrieve.php'; // Required for activation
			//HTML FORM objects
			require_once CN_PATH . '/includes/class.form.php'; // Required for activation
			//date objects
			require_once CN_PATH . '/includes/class.date.php'; // Required for activation, entry list, add entry
			//entry objects
			require_once CN_PATH . '/includes/class.entry.php'; // Required for activation, entry list
			//plugin option objects
			require_once CN_PATH . '/includes/class.options.php'; // Required for activation
			//plugin utility objects
			require_once CN_PATH . '/includes/class.utility.php'; // Required for activation, entry list
			//plugin template objects
			require_once CN_PATH . '/includes/class.output.php'; // Required for activation, entry list
			//builds vCard
			require_once CN_PATH . '/includes/class.vcard.php'; // Required for front end

			// geocoding
			require_once CN_PATH . '/includes/class.geo.php'; // Required

			//shortcodes
			require_once CN_PATH . '/includes/inc.shortcodes.php'; // Required for front end

			//templates
			require_once CN_PATH . '/includes/class.template.php'; // Required for the front end template processing

			// The class that inits the registered query vars, rewites reuls and canonical redirects.
			require_once CN_PATH . '/includes/class.rewrite.php';

			// Load the Connections Settings API Wrapper Class.
			require_once CN_PATH . '/includes/class.settings-api.php';

			// Load the Connections core settings admin page tabs, section and fields using the WordPress Settings API.
			require_once CN_PATH . '/includes/class.settings.php';

			if ( is_admin() ) {
				/*
				 * Include the Screen Options class by Janis Elsts
				 * http://w-shadow.com/blog/2010/06/29/adding-stuff-to-wordpress-screen-options/
				 */
				include CN_PATH . '/includes/screen-options/screen-options.php';
			}

		}

		private function initDependencies() {
			$this->options = new cnOptions();
			$this->settings = cnSettingsAPI::getInstance();
			$this->pageHook = new stdClass();
			$this->currentUser = new cnUser();
			$this->retrieve = new cnRetrieve();
			$this->term = new cnTerms();
			$this->template = new cnTemplate();
			$this->url = new cnURL();

			// Register all valid query variables.
			cnRewrite::init();
		}

		/**
		 * During activation this will initiate the options.
		 */
		private function initOptions() {
			/*
			 * Retrieve the settings stored prior to 0.7.3 and migrate them
			 * so they will be accessible in the structure supported by the
			 * Connections WordPress Settings API Wrapper Class.
			 */
			if ( get_option( 'connections_options' ) !== FALSE ) {
				$options = get_option( 'connections_options' );

				if ( get_option( 'connections_login' ) === FALSE ) {
					update_option( 'connections_login' , array(
							'required' => $options['settings']['allow_public'],
							'message' => 'Please login to view the directory.'
						)
					);
				}

				if ( get_option( 'connections_visibility' ) === FALSE ) {
					update_option( 'connections_visibility' , array(
							'allow_public_override' => $options['settings']['allow_public_override'],
							'allow_private_override' => $options['settings']['allow_private_override']
						)
					);
				}

				if ( get_option( 'connections_image_thumbnail' ) === FALSE ) {
					update_option( 'connections_image_thumbnail' , array(
							'quality' => $options['settings']['image']['thumbnail']['quality'],
							'width' => $options['settings']['image']['thumbnail']['x'],
							'height' => $options['settings']['image']['thumbnail']['y'],
							'ratio' => $options['settings']['image']['thumbnail']['crop']
						)
					);
				}
				if ( get_option( 'connections_image_medium' ) === FALSE ) {
					update_option( 'connections_image_medium' , array(
							'quality' => $options['settings']['image']['entry']['quality'],
							'width' => $options['settings']['image']['entry']['x'],
							'height' => $options['settings']['image']['entry']['y'],
							'ratio' => $options['settings']['image']['entry']['crop']
						)
					);
				}

				if ( get_option( 'connections_image_large' ) === FALSE ) {
					update_option( 'connections_image_large' , array(
							'quality' => $options['settings']['image']['profile']['quality'],
							'width' => $options['settings']['image']['profile']['x'],
							'height' => $options['settings']['image']['profile']['y'],
							'ratio' => $options['settings']['image']['profile']['crop']
						)
					);
				}

				if ( get_option( 'connections_image_logo' ) === FALSE ) {
					update_option( 'connections_image_logo' , array(
							'quality' => $options['settings']['image']['logo']['quality'],
							'width' => $options['settings']['image']['logo']['x'],
							'height' => $options['settings']['image']['logo']['y'],
							'ratio' => $options['settings']['image']['logo']['crop']
						)
					);
				}

				if ( get_option( 'connections_compatibility' ) === FALSE ) {
					update_option( 'connections_compatibility' , array(
							'google_maps_api' => $options['settings']['advanced']['load_google_maps_api'],
							'javascript_footer' => $options['settings']['advanced']['load_javascript_footer'] )
					);
				}

				if ( get_option( 'connections_debug' ) === FALSE ) update_option( 'connections_debug' , array( 'debug_messages' => $options['debug'] ) );

			}

			if ( $this->options->getDefaultTemplatesSet() === NULL ) $this->options->setDefaultTemplates();

			// @TODO: a version change should not reset the roles and capabilites.
			$this->options->setDefaultCapabilities();

			// Increment the version number.
			$this->options->setVersion( CN_CURRENT_VERSION );

			// Save the options
			$this->options->saveOptions();

			/*
			 * This option is added for a check that will force a flush_rewrite() in connectionsLoad::adminInit() once.
			 * Should save the user from having to "save" the permalink settings.
			 */
			update_option( 'connections_flush_rewrite', '1' );
		}

		public function displayMessages() {
			// Exit the method if $_GET['display_messages'] isn't set.
			// if ( ! isset( $_GET['display_messages'] ) ) return;

			$output = '';

			$messages = $this->currentUser->getMessages();

			if ( !empty( $messages ) ) {
				foreach ( $messages as $message ) {
					foreach ( $message as $type => $code ) {
						switch ( $type ) {
						case 'error':
							$output .= '<div id="message" class="error"><p><strong>' . __( 'ERROR', 'connections' ) . ': </strong>' . $this->message->get_error_message( $code ) . '</p></div>';
							break;

						case 'error_runtime':
							$output .= '<div id="message" class="error"><p><strong>' . __( 'ERROR', 'connections' ) . ': </strong>' . $code . '</p></div>';
							break;

						case 'success':
							$output .= '<div id="message" class="updated fade"><p><strong>' . __( 'SUCCESS', 'connections' ) . ': </strong>' . $this->message->get_error_message( $code ) . '</p></div>';
							break;

						case 'success_runtime':
							$output .= '<div id="message" class="updated fade"><p><strong>' . __( 'SUCCESS', 'connections' ) . ': </strong>' . $code . '</p></div>';
							break;
						}
					}
				}
			}

			/*if ( $_GET['display_messages'] === 'true' )*/ $this->currentUser->resetMessages();

			echo $output;
		}

		public function setRuntimeMessage( $type , $message ) {
			$messages = $this->currentUser->getMessages();

			$this->currentUser->setMessage( array( $type => $message ) );
		}

		/**
		 * Initiate the error messages for Connections using the WP_Error class.
		 */
		private function initErrorMessages() {
			$this->message->add( 'capability_view_entry_list', __( 'You are not authorized to view the entry list. Please contact the admin if you received this message in error.', 'connections' ) );

			$this->message->add( 'capability_add', __( 'You are not authorized to add entries. Please contact the admin if you received this message in error.', 'connections' ) );
			$this->message->add( 'capability_delete', __( 'You are not authorized to delete entries. Please contact the admin if you received this message in error.', 'connections' ) );
			$this->message->add( 'capability_edit', __( 'You are not authorized to edit entries. Please contact the admin if you received this message in error.', 'connections' ) );
			$this->message->add( 'capability_categories', __( 'You are not authorized to edit the categories. Please contact the admin if you received this message in error.', 'connections' ) );
			$this->message->add( 'capability_settings', __( 'You are not authorized to edit the settings. Please contact the admin if you received this message in error.', 'connections' ) );
			$this->message->add( 'capability_roles', __( 'You are not authorized to edit role capabilities. Please contact the admin if you received this message in error.', 'connections' ) );

			$this->message->add( 'category_duplicate_name', __( 'The category you are trying to create already exists.', 'connections' ) );
			$this->message->add( 'category_self_parent', __( 'Category can not be a parent of itself.', 'connections' ) );
			$this->message->add( 'category_delete_uncategorized', __( 'The Uncategorized category can not be deleted.', 'connections' ) );
			$this->message->add( 'category_update_uncategorized', __( 'The Uncategorized category can not be altered.', 'connections' ) );
			$this->message->add( 'category_add_uncategorized', __( 'The Uncategorized category already exists.', 'connections' ) );
			$this->message->add( 'category_add_failed', __( 'Failed to add category.', 'connections' ) );
			$this->message->add( 'category_update_failed', __( 'Failed to update category.', 'connections' ) );
			$this->message->add( 'category_delete_failed', __( 'Failed to delete category.', 'connections' ) );

			$this->message->add( 'entry_added_failed', __( 'Entry could not be added.', 'connections' ) );
			$this->message->add( 'entry_updated_failed', __( 'Entry could not be updated.', 'connections' ) );

			$this->message->add( 'entry_preferred_overridden_address', __( 'Your preferred setting for a address was overridden because another address that you are not permitted to view or edit is set as the preferred address. Please contact the admin if you received this message in error.', 'connections' ) );
			$this->message->add( 'entry_preferred_overridden_phone', __( 'Your preferred setting for a phone was overridden because another phone number that you are not permitted to view or edit is set as the preferred phone number. Please contact the admin if you received this message in error.', 'connections' ) );
			$this->message->add( 'entry_preferred_overridden_email', __( 'Your preferred setting for an email address was overridden because another email address that you are not permitted to view or edit is set as the preferred email address. Please contact the admin if you received this message in error.', 'connections' ) );
			$this->message->add( 'entry_preferred_overridden_im', __( 'Your preferred setting for a IM Network was overridden because another IM Network that you are not permitted to view or edit is set as the preferred IM Network. Please contact the admin if you received this message in error.', 'connections' ) );
			$this->message->add( 'entry_preferred_overridden_social', __( 'Your preferred setting for a social media network was overridden because another social media network that you are not permitted to view or edit is set as the preferred social media network. Please contact the admin if you received this message in error.', 'connections' ) );
			$this->message->add( 'entry_preferred_overridden_link', __( 'Your preferred setting for a link was overridden because another link that you are not permitted to view or edit is set as the preferred link. Please contact the admin if you received this message in error.', 'connections' ) );
			$this->message->add( 'entry_preferred_overridden_date', __( 'Your preferred setting for a date was overridden because another date that you are not permitted to view or edit is set as the preferred date. Please contact the admin if you received this message in error.', 'connections' ) );

			$this->message->add( 'image_upload_failed', __( 'Image upload failed.', 'connections' ) );
			$this->message->add( 'image_uploaded_failed', __( 'Uploaded image could not be saved to the destination folder.', 'connections' ) );
			$this->message->add( 'image_profile_failed', __( 'Profile image could not be created and/or saved to the destination folder.', 'connections' ) );
			$this->message->add( 'image_entry_failed', __( 'Entry image could not be created and/or saved to the destination folder.', 'connections' ) );
			$this->message->add( 'image_thumbnail_failed', __( 'Thumbnail image could not be created and/or saved to the destination folder.', 'connections' ) );

			$this->message->add( 'template_install_failed', __( 'The template installation has failed.', 'connections' ) );
			$this->message->add( 'template_delete_failed', __( 'The template could not be deleted.', 'connections' ) );
		}

		/**
		 * Returns one of the predefined Connections error messages.
		 *
		 * @return HTML div with error message.
		 * @param string
		 */
		public function getErrorMessage( $errorMessage ) {
			return '<div id="message" class="error"><p><strong>' . __( 'ERROR', 'connections' ) . ': </strong>' . $this->message->get_error_message( $errorMessage ) . '</p></div>';
		}

		/**
		 * Stores a predefined error messages in the user meta.
		 *
		 * @param string
		 */
		public function setErrorMessage( $errorMessage ) {
			$messages = $this->currentUser->getMessages();

			// If the error message is already stored, no need to store it twice.
			if ( ! in_array( array( 'error' => $errorMessage ) , $messages ) ) $this->currentUser->setMessage( array( 'error' => $errorMessage ) );
		}

		/**
		 * Initiates the success messages for Connections using the WP_Error class.
		 */
		private function initSuccessMessages() {
			$this->message->add( 'form_entry_delete', __( 'The entry has been deleted.', 'connections' ) );
			$this->message->add( 'form_entry_delete_bulk', __( 'Entry(ies) have been deleted.', 'connections' ) );
			$this->message->add( 'form_entry_pending', __( 'The entry status have been set to pending.', 'connections' ) );
			$this->message->add( 'form_entry_pending_bulk', __( 'Entry(ies) status have been set to pending.', 'connections' ) );
			$this->message->add( 'form_entry_approve', __( 'The entry has been approved.', 'connections' ) );
			$this->message->add( 'form_entry_approve_bulk', __( 'Entry(ies) have been approved.', 'connections' ) );
			$this->message->add( 'form_entry_visibility_bulk', __( 'Entry(ies) visibility have been updated.', 'connections' ) );

			$this->message->add( 'category_deleted', __( 'Category(ies) have been deleted.', 'connections' ) );
			$this->message->add( 'category_updated', __( 'Category has been updated.', 'connections' ) );
			$this->message->add( 'category_added', __( 'Category has been added.', 'connections' ) );

			$this->message->add( 'entry_added', __( 'Entry has been added.', 'connections' ) );
			$this->message->add( 'entry_added_moderated', __( 'Pending review entry will be added.', 'connections' ) );
			$this->message->add( 'entry_updated', __( 'Entry has been updated.', 'connections' ) );
			$this->message->add( 'entry_updated_moderated', __( 'Pending review entry will be updated.', 'connections' ) );

			$this->message->add( 'image_uploaded', __( 'Uploaded image saved.', 'connections' ) );
			$this->message->add( 'image_profile', __( 'Profile image created and saved.', 'connections' ) );
			$this->message->add( 'image_entry', __( 'Entry image created and saved.', 'connections' ) );
			$this->message->add( 'image_thumbnail', __( 'Thumbnail image created and saved.', 'connections' ) );

			$this->message->add( 'role_settings_updated', __( 'Role capabilities have been updated.', 'connections' ) );

			$this->message->add( 'template_change_active', __( 'The default active template has been changed.', 'connections' ) );
			$this->message->add( 'template_installed', __( 'A new template has been installed.', 'connections' ) );
			$this->message->add( 'template_deleted', __( 'The template has been deleted.', 'connections' ) );
		}

		/**
		 * Returns one of the predefined Connections success messages.
		 *
		 * @return HTML div with error message.
		 * @param string
		 */
		public function getSuccessMessage( $successMessage ) {
			return '<div id="message" class="updated fade"><p><strong>' . __( 'SUCCESS', 'connections' ) . ': </strong>' . $this->message->get_error_message( $successMessage ) . '</p></div>';
		}

		/**
		 * Stores a predefined success messages in the user meta.
		 *
		 * @param string
		 */
		public function setSuccessMessage( $successMessage ) {
			$messages = $this->currentUser->getMessages();

			// If the success message is slready stored, no need to store it twice.
			if ( ! in_array( array( 'success' => $successMessage ) , $messages ) ) $this->currentUser->setMessage( array( 'success' => $successMessage ) );
		}

		/**
		 * Called when activating Connections via the activation hook.
		 */
		public function activate() {
			global $wpdb, $connections;
			require_once ABSPATH . 'wp-admin/includes/upgrade.php';

			$charsetCollate = '';

			if ( ! empty( $wpdb->charset ) ) $charsetCollate = "DEFAULT CHARACTER SET $wpdb->charset";
			if ( ! empty( $wpdb->collate ) ) $charsetCollate .= " COLLATE $wpdb->collate";


			if ( $wpdb->get_var( "SHOW TABLES LIKE '" . CN_ENTRY_TABLE . "'" ) != CN_ENTRY_TABLE ) {
				$entryTable = "CREATE TABLE " . CN_ENTRY_TABLE . " (
			        id bigint(20) NOT NULL AUTO_INCREMENT,
			        ts TIMESTAMP,
					date_added tinytext NOT NULL,
			        entry_type tinytext NOT NULL,
					visibility tinytext NOT NULL,
					slug tinytext NOT NULL,
					family_name tinytext NOT NULL,
					honorific_prefix tinytext NOT NULL,
					first_name tinytext NOT NULL,
					middle_name tinytext NOT NULL,
			        last_name tinytext NOT NULL,
					honorific_suffix tinytext NOT NULL,
					title tinytext NOT NULL,
					organization tinytext NOT NULL,
					department tinytext NOT NULL,
					contact_first_name tinytext NOT NULL,
					contact_last_name tinytext NOT NULL,
					addresses longtext NOT NULL,
					phone_numbers longtext NOT NULL,
					email longtext NOT NULL,
					im longtext NOT NULL,
					social longtext NOT NULL,
					links longtext NOT NULL,
					dates longtext NOT NULL,
					birthday tinytext NOT NULL,
					anniversary tinytext NOT NULL,
					bio longtext NOT NULL,
			        notes longtext NOT NULL,
					options longtext NOT NULL,
					added_by bigint(20) NOT NULL,
					edited_by bigint(20) NOT NULL,
					owner bigint(20) NOT NULL,
					user bigint(20) NOT NULL,
					status varchar(20) NOT NULL,
			        PRIMARY KEY  (id)
			    ) $charsetCollate";

				// Create the table
				dbDelta( $entryTable );

				/*
				 * Alter the table after is was created to add FULLTEXT support.
				 * That way if the db engine doesn't support it, at least the table will be created.
				 */
				$wpdb->query( 'ALTER TABLE ' . CN_ENTRY_TABLE . ' ADD FULLTEXT search (family_name, first_name, middle_name, last_name, title, organization, department, contact_first_name, contact_last_name, bio, notes)' );
			}

			if ( $wpdb->get_var( "SHOW TABLES LIKE '" . CN_TERMS_TABLE . "'" ) != CN_TERMS_TABLE ) {
				$termsTable = "CREATE TABLE " . CN_TERMS_TABLE . " (
			        term_id bigint(20) NOT NULL AUTO_INCREMENT,
					name varchar(200) NOT NULL,
					slug varchar(200) NOT NULL,
					term_group bigint(10) NOT NULL,
			        PRIMARY KEY  (term_id),
					UNIQUE KEY slug (slug),
					INDEX name (name)
			    ) $charsetCollate";

				// Create the table
				dbDelta( $termsTable );
			}

			if ( $wpdb->get_var( "SHOW TABLES LIKE '" . CN_TERM_TAXONOMY_TABLE . "'" ) != CN_TERM_TAXONOMY_TABLE ) {
				$termTaxonomyTable = "CREATE TABLE " . CN_TERM_TAXONOMY_TABLE . " (
			        term_taxonomy_id bigint(20) NOT NULL AUTO_INCREMENT,
					term_id bigint(20) NOT NULL,
					taxonomy varchar(32) NOT NULL,
					description longtext NOT NULL,
					parent bigint(20) NOT NULL,
					count bigint(20) NOT NULL,
			        PRIMARY KEY  (term_taxonomy_id),
					UNIQUE KEY term_id_taxonomy (term_id, taxonomy),
					INDEX taxonomy (taxonomy)
			    ) $charsetCollate";

				// Create the table
				dbDelta( $termTaxonomyTable );
			}

			if ( $wpdb->get_var( "SHOW TABLES LIKE '" . CN_TERM_RELATIONSHIP_TABLE . "'" ) != CN_TERM_RELATIONSHIP_TABLE ) {
				$termTermRelationshipTable = "CREATE TABLE " . CN_TERM_RELATIONSHIP_TABLE . " (
			        entry_id bigint(20) NOT NULL,
					term_taxonomy_id bigint(20) NOT NULL,
					term_order int(11) NOT NULL,
			        PRIMARY KEY (entry_id,term_taxonomy_id),
					KEY term_taxonomy_id (term_taxonomy_id)
			    ) $charsetCollate";

				// Create the table
				dbDelta( $termTermRelationshipTable );
			}


			// Check if the Uncategorized term exists and if it doesn't create it.
			$term = $connections->term->getTermBy( 'slug', 'uncategorized', 'category' );

			if ( empty( $term ) ) {
				$attributes['slug'] = '';
				$attributes['parent'] = 0;
				$attributes['description'] = __( 'Entries not assigned to a category will automatically be assigned to this category and deleting a category which has been assigned to an entry will reassign that entry to this category. This category can not be edited or deleted.', 'connections' ) ;

				$connections->term->addTerm( __( 'Uncategorized', 'connections' ) , 'category', $attributes );
			}


			if ( $wpdb->get_var( "SHOW TABLES LIKE '" . CN_ENTRY_TABLE_META . "'" ) != CN_ENTRY_TABLE_META ) {
				$entryTableMeta = "CREATE TABLE " . CN_ENTRY_TABLE_META . " (
			        meta_id bigint(20) unsigned NOT NULL auto_increment,
					entry_id bigint(20) unsigned NOT NULL default '0',
					meta_key varchar(255) default NULL,
					meta_value longtext,
					PRIMARY KEY  (meta_id),
					KEY entry_id (entry_id),
					KEY meta_key (meta_key)
			    ) $charsetCollate";

				// Create the table
				dbDelta( $entryTableMeta );
			}

			if ( $wpdb->get_var( "SHOW TABLES LIKE '" . CN_ENTRY_ADDRESS_TABLE . "'" ) != CN_ENTRY_ADDRESS_TABLE ) {
				require_once ABSPATH . 'wp-admin/includes/upgrade.php';

				$entryTableAddress = "CREATE TABLE " . CN_ENTRY_ADDRESS_TABLE . " (
			        `id` bigint(20) unsigned NOT NULL auto_increment,
					`entry_id` bigint(20) unsigned NOT NULL default '0',
					`order` tinyint unsigned NOT NULL default '0',
					`preferred` tinyint unsigned NOT NULL default '0',
					`type` tinytext NOT NULL,
					`line_1` tinytext NOT NULL,
					`line_2` tinytext NOT NULL,
					`line_3` tinytext NOT NULL,
					`city` tinytext NOT NULL,
					`state` tinytext NOT NULL,
					`zipcode` tinytext NOT NULL,
					`country` tinytext NOT NULL,
					`latitude` decimal(15,12) default NULL,
					`longitude` decimal(15,12) default NULL,
					`visibility` tinytext NOT NULL,
					PRIMARY KEY (`id`, `entry_id`)
			    ) $charsetCollate";

				// Create the table
				dbDelta( $entryTableAddress );

				/*
				 * Alter the table after is was created to add FULLTEXT support.
				 * That way if the db engine doesn't support it, at least the table will be created.
				 */
				$wpdb->query( 'ALTER TABLE ' . CN_ENTRY_ADDRESS_TABLE . ' ADD FULLTEXT search (line_1, line_2, line_3, city, state, zipcode, country)' );
			}

			if ( $wpdb->get_var( "SHOW TABLES LIKE '" . CN_ENTRY_PHONE_TABLE . "'" ) != CN_ENTRY_PHONE_TABLE ) {
				require_once ABSPATH . 'wp-admin/includes/upgrade.php';

				$entryTablePhone = "CREATE TABLE " . CN_ENTRY_PHONE_TABLE . " (
			        `id` bigint(20) unsigned NOT NULL auto_increment,
					`entry_id` bigint(20) unsigned NOT NULL default '0',
					`order` tinyint unsigned NOT NULL default '0',
					`preferred` tinyint unsigned NOT NULL default '0',
					`type` tinytext NOT NULL,
					`number` tinytext NOT NULL,
					`visibility` tinytext NOT NULL,
					PRIMARY KEY (`id`, `entry_id`)
			    ) $charsetCollate";

				// Create the table
				dbDelta( $entryTablePhone );

				/*
				 * Alter the table after is was created to add FULLTEXT support.
				 * That way if the db engine doesn't support it, at least the table will be created.
				 */
				$wpdb->query( 'ALTER TABLE ' . CN_ENTRY_PHONE_TABLE . ' ADD FULLTEXT search (number)' );
			}

			if ( $wpdb->get_var( "SHOW TABLES LIKE '" . CN_ENTRY_EMAIL_TABLE . "'" ) != CN_ENTRY_EMAIL_TABLE ) {
				require_once ABSPATH . 'wp-admin/includes/upgrade.php';

				$entryTableEmail = "CREATE TABLE " . CN_ENTRY_EMAIL_TABLE . " (
			        `id` bigint(20) unsigned NOT NULL auto_increment,
					`entry_id` bigint(20) unsigned NOT NULL default '0',
					`order` tinyint unsigned NOT NULL default '0',
					`preferred` tinyint unsigned NOT NULL default '0',
					`type` tinytext NOT NULL,
					`address` tinytext NOT NULL,
					`visibility` tinytext NOT NULL,
					PRIMARY KEY (`id`, `entry_id`)
			    ) $charsetCollate";

				// Create the table
				dbDelta( $entryTableEmail );
			}

			if ( $wpdb->get_var( "SHOW TABLES LIKE '" . CN_ENTRY_MESSENGER_TABLE . "'" ) != CN_ENTRY_MESSENGER_TABLE ) {
				require_once ABSPATH . 'wp-admin/includes/upgrade.php';

				$entryTableMessenger = "CREATE TABLE " . CN_ENTRY_MESSENGER_TABLE . " (
			        `id` bigint(20) unsigned NOT NULL auto_increment,
					`entry_id` bigint(20) unsigned NOT NULL default '0',
					`order` tinyint unsigned NOT NULL default '0',
					`preferred` tinyint unsigned NOT NULL default '0',
					`type` tinytext NOT NULL,
					`uid` tinytext NOT NULL,
					`visibility` tinytext NOT NULL,
					PRIMARY KEY (`id`, `entry_id`)
			    ) $charsetCollate";

				// Create the table
				dbDelta( $entryTableMessenger );
			}

			if ( $wpdb->get_var( "SHOW TABLES LIKE '" . CN_ENTRY_SOCIAL_TABLE . "'" ) != CN_ENTRY_SOCIAL_TABLE ) {
				require_once ABSPATH . 'wp-admin/includes/upgrade.php';

				$entryTableSocialMedia = "CREATE TABLE " . CN_ENTRY_SOCIAL_TABLE . " (
			        `id` bigint(20) unsigned NOT NULL auto_increment,
					`entry_id` bigint(20) unsigned NOT NULL default '0',
					`order` tinyint unsigned NOT NULL default '0',
					`preferred` tinyint unsigned NOT NULL default '0',
					`type` tinytext NOT NULL,
					`url` tinytext NOT NULL,
					`visibility` tinytext NOT NULL,
					PRIMARY KEY (`id`, `entry_id`)
			    ) $charsetCollate";

				// Create the table
				dbDelta( $entryTableSocialMedia );
			}

			if ( $wpdb->get_var( "SHOW TABLES LIKE '" . CN_ENTRY_LINK_TABLE . "'" ) != CN_ENTRY_LINK_TABLE ) {
				require_once ABSPATH . 'wp-admin/includes/upgrade.php';

				$entryTableLink = "CREATE TABLE " . CN_ENTRY_LINK_TABLE . " (
			        `id` bigint(20) unsigned NOT NULL auto_increment,
					`entry_id` bigint(20) unsigned NOT NULL default '0',
					`order` tinyint unsigned NOT NULL default '0',
					`preferred` tinyint unsigned NOT NULL default '0',
					`type` tinytext NOT NULL,
					`title` tinytext NOT NULL,
					`url` tinytext NOT NULL,
					`target` tinytext NOT NULL,
					`follow` tinyint unsigned NOT NULL default '0',
					`image` tinyint unsigned NOT NULL default '0',
					`logo` tinyint unsigned NOT NULL default '0',
					`visibility` tinytext NOT NULL,
					PRIMARY KEY (`id`, `entry_id`)
			    ) $charsetCollate";

				// Create the table
				dbDelta( $entryTableLink );
			}

			if ( $wpdb->get_var( "SHOW TABLES LIKE '" . CN_ENTRY_DATE_TABLE . "'" ) != CN_ENTRY_DATE_TABLE ) {
				require_once ABSPATH . 'wp-admin/includes/upgrade.php';

				$entryTableDate = "CREATE TABLE " . CN_ENTRY_DATE_TABLE . " (
			        `id` bigint(20) unsigned NOT NULL auto_increment,
					`entry_id` bigint(20) unsigned NOT NULL default '0',
					`order` tinyint unsigned NOT NULL default '0',
					`preferred` tinyint unsigned NOT NULL default '0',
					`type` tinytext NOT NULL,
					`date` date NOT NULL default '0000-00-00',
					`visibility` tinytext NOT NULL,
					PRIMARY KEY (`id`, `entry_id`)
			    ) $charsetCollate";

				// Create the table
				dbDelta( $entryTableDate );
			}

			// Create the cache folder.
			wp_mkdir_p( CN_CACHE_PATH );
			//if ( ! file_exists( CN_CACHE_PATH ) ) wp_mkdir_p( CN_CACHE_PATH );

			/*
			 * Attempt to set the folder writeable per http://codex.wordpress.org/Changing_File_Permissions#Using_the_Command_Line
			 */
			if ( file_exists( CN_CACHE_PATH ) && ! is_writeable( CN_CACHE_PATH ) ) @chmod( CN_CACHE_PATH , 0746 );
			if ( file_exists( CN_CACHE_PATH ) && ! is_writeable( CN_CACHE_PATH ) ) @chmod( CN_CACHE_PATH , 0747 );
			if ( file_exists( CN_CACHE_PATH ) && ! is_writeable( CN_CACHE_PATH ) ) @chmod( CN_CACHE_PATH , 0756 );
			if ( file_exists( CN_CACHE_PATH ) && ! is_writeable( CN_CACHE_PATH ) ) @chmod( CN_CACHE_PATH , 0757 );
			if ( file_exists( CN_CACHE_PATH ) && ! is_writeable( CN_CACHE_PATH ) ) @chmod( CN_CACHE_PATH , 0764 );
			if ( file_exists( CN_CACHE_PATH ) && ! is_writeable( CN_CACHE_PATH ) ) @chmod( CN_CACHE_PATH , 0765 );
			if ( file_exists( CN_CACHE_PATH ) && ! is_writeable( CN_CACHE_PATH ) ) @chmod( CN_CACHE_PATH , 0766 );
			if ( file_exists( CN_CACHE_PATH ) && ! is_writeable( CN_CACHE_PATH ) ) @chmod( CN_CACHE_PATH , 0767 );

			// Create the images folder.
			wp_mkdir_p( CN_IMAGE_PATH );

			/*
			 * Attempt to set the folder writeable per http://codex.wordpress.org/Changing_File_Permissions#Using_the_Command_Line
			 */
			if ( file_exists( CN_IMAGE_PATH ) && ! is_writeable( CN_IMAGE_PATH ) ) @chmod( CN_IMAGE_PATH , 0746 );
			if ( file_exists( CN_IMAGE_PATH ) && ! is_writeable( CN_IMAGE_PATH ) ) @chmod( CN_IMAGE_PATH , 0747 );
			if ( file_exists( CN_IMAGE_PATH ) && ! is_writeable( CN_IMAGE_PATH ) ) @chmod( CN_IMAGE_PATH , 0756 );
			if ( file_exists( CN_IMAGE_PATH ) && ! is_writeable( CN_IMAGE_PATH ) ) @chmod( CN_IMAGE_PATH , 0757 );
			if ( file_exists( CN_IMAGE_PATH ) && ! is_writeable( CN_IMAGE_PATH ) ) @chmod( CN_IMAGE_PATH , 0764 );
			if ( file_exists( CN_IMAGE_PATH ) && ! is_writeable( CN_IMAGE_PATH ) ) @chmod( CN_IMAGE_PATH , 0765 );
			if ( file_exists( CN_IMAGE_PATH ) && ! is_writeable( CN_IMAGE_PATH ) ) @chmod( CN_IMAGE_PATH , 0766 );
			if ( file_exists( CN_IMAGE_PATH ) && ! is_writeable( CN_IMAGE_PATH ) ) @chmod( CN_IMAGE_PATH , 0767 );

			// Create the custom template folder.
			wp_mkdir_p( CN_CUSTOM_TEMPLATE_PATH );

			/*
			 * Attempt to set the folder writeable per http://codex.wordpress.org/Changing_File_Permissions#Using_the_Command_Line
			 */
			if ( file_exists( CN_CUSTOM_TEMPLATE_PATH ) && ! is_writeable( CN_CUSTOM_TEMPLATE_PATH ) ) @chmod( CN_CUSTOM_TEMPLATE_PATH , 0746 );
			if ( file_exists( CN_CUSTOM_TEMPLATE_PATH ) && ! is_writeable( CN_CUSTOM_TEMPLATE_PATH ) ) @chmod( CN_CUSTOM_TEMPLATE_PATH , 0747 );
			if ( file_exists( CN_CUSTOM_TEMPLATE_PATH ) && ! is_writeable( CN_CUSTOM_TEMPLATE_PATH ) ) @chmod( CN_CUSTOM_TEMPLATE_PATH , 0756 );
			if ( file_exists( CN_CUSTOM_TEMPLATE_PATH ) && ! is_writeable( CN_CUSTOM_TEMPLATE_PATH ) ) @chmod( CN_CUSTOM_TEMPLATE_PATH , 0757 );
			if ( file_exists( CN_CUSTOM_TEMPLATE_PATH ) && ! is_writeable( CN_CUSTOM_TEMPLATE_PATH ) ) @chmod( CN_CUSTOM_TEMPLATE_PATH , 0764 );
			if ( file_exists( CN_CUSTOM_TEMPLATE_PATH ) && ! is_writeable( CN_CUSTOM_TEMPLATE_PATH ) ) @chmod( CN_CUSTOM_TEMPLATE_PATH , 0765 );
			if ( file_exists( CN_CUSTOM_TEMPLATE_PATH ) && ! is_writeable( CN_CUSTOM_TEMPLATE_PATH ) ) @chmod( CN_CUSTOM_TEMPLATE_PATH , 0766 );
			if ( file_exists( CN_CUSTOM_TEMPLATE_PATH ) && ! is_writeable( CN_CUSTOM_TEMPLATE_PATH ) ) @chmod( CN_CUSTOM_TEMPLATE_PATH , 0767 );

			$this->initOptions();

			/*
			 * Add the page rewrite rules.
			 */
			add_filter( 'root_rewrite_rules', array( $this, 'addRootRewriteRules' ) );
			add_filter( 'page_rewrite_rules', array( $this, 'addPageRewriteRules' ) );

			// Flush so they are rebuilt.
			flush_rewrite_rules();
		}

		/**
		 * Called when deactivating Connections via the deactivation hook.
		 */
		public function deactivate() {
			/*
			 * Since we're adding the rewrite rules using a filter, make sure to remove the filter
			 * before flushing, otherwise the rules will not be removed.
			 */
			remove_filter( 'root_rewrite_rules', array( $this, 'addRootRewriteRules' ) );
			remove_filter( 'page_rewrite_rules', array( $this, 'addPageRewriteRules' ) );

			// Flush so they are rebuilt.
			flush_rewrite_rules();

			//global $options;

			/* This should be occur in the unistall hook
			$this->options->removeDefaultCapabilities();
			*/

			//  DROP TABLE `cnpfresh_connections`, `cnpfresh_connections_terms`, `cnpfresh_connections_term_relationships`, `cnpfresh_connections_term_taxonomy`;
			//  DELETE FROM `nhonline_freshcnpro`.`cnpfresh_options` WHERE `cnpfresh_options`.`option_name` = 'connections_options'
		}

		public function addDBUpgradeMessage() {
			echo '<div id="message" class="error"><p><strong>' . __( 'Connections database requires updating.', 'connections' )  . '<a class="button-highlighted" href="admin.php?page=connections_manage">' . __( 'START', 'connections' )  . '</a></strong></p></div>';
		}

		/**
		 * Remove the admin notice since it's confusing to see both the message and upgrade text at the same time.
		 * Called in action toplevel_page_connections_dashboard from adminInit()
		 */
		public function removeDBUpgradeMessage() {
			remove_action( 'admin_notices', array( &$this , 'addDBUpgradeMessage' ) );
		}

		/**
		 * Add the rewrite rules.
		 * NOTE: No action is set to call this function.
		 *
		 * @access private
		 * @since 0.7.3
		 * @uses flush_rewrite_rules()
		 * @uses add_rewrite_rule()
		 * @return void
		 */
		public function addRewriteRules() {
			/* TESTING FOR VITURAL PAGES ONLY */
			// base URI
			add_rewrite_rule( '(directory)/?$', 'index.php?cnpagename=$matches[1]', 'top' );
			add_rewrite_rule( '(directory)/(individual|organization|family)/?$', 'index.php?cnpagename=$matches[1]&cnlt=$matches[2]', 'top' );
			add_rewrite_rule( '(directory)/name/(.*)/?$', 'index.php?cnpagename=$matches[1]&cnname=$matches[2]', 'top' );

			//add_rewrite_rule( '(directory)/?([^/]*)/?([^/]*)$', 'index.php?cnpagename=$matches[1]&cnlisttype=$matches[2]&cnname=$matches[3]', 'top' );

			/* For testing only, NEVER leave uncommented in released versions. It'll slow WP down. */
			// flush_rewrite_rules();
		}

		/**
		 * Initialize the admin.
		 *
		 * @access private
		 * @since unknown
		 * @uses WP_Error()
		 * @uses get_option()
		 * @uses delete_option()
		 * @uses add_action()
		 * @uses add_filter()
		 * @uses add_screen_options_panel()
		 * @return void
		 */
		public function adminInit() {
			$directoryHome = $this->settings->get( 'connections', 'connections_home_page', 'page_id' );

			// Initiate admin messages.
			$this->message = new WP_Error();
			$this->initErrorMessages();
			$this->initSuccessMessages();

			// If the user changed the base slugs for the permalinks, flush the rewrite rules.
			if ( get_option( 'connections_flush_rewrite' ) ) {
				flush_rewrite_rules();
				delete_option( 'connections_flush_rewrite' );
			}

			/*
			 * If the home page has not been set, nag the user to set it.
			 */
			if ( ! $directoryHome ) add_action(
					'admin_notices',
					create_function(
						'',
						'echo \'<div id="message" class="error"><p>' . __( '<strong>ERROR:</strong> The Connections directory home page has not been set. Please set it now on the Connections : Seetings page under the General tab.', 'connections' ) . '</p></div>\';'
					)
				);

			//Check if the db requires updating, display message if it does.
			if ( version_compare( $this->options->getDBVersion() , CN_DB_VERSION ) < 0 ) $this->dbUpgrade = add_action( 'admin_notices' , array( $this , 'addDBUpgradeMessage' ) );

			/*
			 * Add admin notices if required directories are not present or not writeable.
			 */
			if ( ! file_exists( CN_IMAGE_PATH ) ) add_action( 'admin_notices' , create_function( '' , ' echo \'<div id="message" class="error"><p>' . __( '<strong>ERROR:</strong> Path ../wp-content/connection_images does not seem to exist. Please try deactivating and reactivating Connections.', 'connections' ) . '</p></div>\';' ) );
			if ( file_exists( CN_IMAGE_PATH ) && ! is_writeable( CN_IMAGE_PATH ) ) add_action( 'admin_notices' , create_function( '' , ' echo \'<div id="message" class="error"><p>' . __( '<strong>ERROR:</strong> Path ../wp-content/connection_images does not seem to be writeable.', 'connections' ) . '</p></div>\';' ) );
			if ( ! file_exists( CN_CUSTOM_TEMPLATE_PATH ) ) add_action( 'admin_notices' , create_function( '' , ' echo \'<div id="message" class="error"><p>' . __( '<strong>ERROR:</strong> Path ../wp-content/connections_templates does not seem to exist. Please try deactivating and reactivating Connections.', 'connections' ) . '</p></div>\';' ) );
			if ( file_exists( CN_CUSTOM_TEMPLATE_PATH ) && ! is_writeable( CN_CUSTOM_TEMPLATE_PATH ) ) add_action( 'admin_notices' , create_function( '' , ' echo \'<div id="message" class="error"><p>' . __( '<strong>ERROR:</strong> Path ../wp-content/connections_templates does not seem to be writeable.', 'connections' ) . '</p></div>\';' ) );
			if ( ! file_exists( CN_CACHE_PATH ) ) add_action( 'admin_notices' , create_function( '' , ' echo \'<div id="message" class="error"><p>' . __( '<strong>ERROR:</strong> Path ../wp-content/plugins/connections/cache does not seem to exist. Please try deactivating and reactivating Connections.', 'connections' ) . '</p></div>\';' ) );
			if ( file_exists( CN_CACHE_PATH ) && ! is_writeable( CN_CACHE_PATH ) ) add_action( 'admin_notices' , create_function( '' , ' echo \'<div id="message" class="error"><p>' . __( '<strong>ERROR:</strong> Path ../wp-content/plugins/connections/cache does not seem to be writeable.', 'connections' ) . '</p></div>\';' ) );

			// Calls the methods to load the admin scripts and CSS.
			add_action( 'admin_enqueue_scripts', array( __CLASS__, 'enqueueAdminScripts' ) );
			add_action( 'admin_enqueue_scripts', array( __CLASS__, 'enqueueAdminStyles' ) );

			// Add Settings link to the plugin actions
			add_action( 'plugin_action_links_' . CN_BASE_NAME, array( $this, 'addActionLinks' ) );

			// Add FAQ, Support and Donate links
			add_filter( 'plugin_row_meta', array( $this, 'addMetaLinks' ), 10, 2 );

			// Add Changelog table row in the Manage Plugins admin page.
			add_action( 'after_plugin_row_' . CN_BASE_NAME, array( $this, 'displayUpgradeNotice' ), 1, 0 );
			// Maybe should use this action hook instead: in_plugin_update_message-{$file}

			/*
			 * In instances such as WP AJAX requests the add_menu() and add_sub_menu() functions are
			 * not run in the admin_menu action, so the properties would not exist and will throw
			 * PHP notices when attempting to access them. If the menus have been added then the
			 * properties will exist so it will be safe to add the actions using the properties.
			 */
			if ( get_object_vars( $this->pageHook ) && current_user_can( 'connections_view_menu') ) {
				// Register the edit metaboxes.
				add_action( 'load-' . $this->pageHook->add, array( $this, 'registerEditMetaboxes' ) );
				add_action( 'load-' . $this->pageHook->manage, array( $this, 'registerEditMetaboxes' ) );

				// Register the Dashboard metaboxes.
				add_action( 'load-' . $this->pageHook->dashboard, array( $this, 'registerDashboardMetaboxes' ) );

				// Remove the admin database message on the following pages since it's confusing to see both the message and upgrade text at the same time.
				add_action( 'load-' . $this->pageHook->dashboard, array( $this, 'removeDBUpgradeMessage' ) );
				add_action( 'load-' . $this->pageHook->manage, array( $this, 'removeDBUpgradeMessage' ) );
				add_action( 'load-' . $this->pageHook->add, array( $this, 'removeDBUpgradeMessage' ) );
				add_action( 'load-' . $this->pageHook->categories, array( $this, 'removeDBUpgradeMessage' ) );
				add_action( 'load-' . $this->pageHook->templates, array( $this, 'removeDBUpgradeMessage' ) );
				add_action( 'load-' . $this->pageHook->settings, array( $this, 'removeDBUpgradeMessage' ) );
				add_action( 'load-' . $this->pageHook->roles, array( $this, 'removeDBUpgradeMessage' ) );

				/*
				 * Add the panel to the "Screen Options" box to the manage page.
				 * NOTE: This relies on the the Screen Options class by Janis Elsts
				 */
				add_screen_options_panel( 'cn-manage-page-limit' , 'Show on screen' , array( $this, 'managePageLimit' ) , $this->pageHook->manage , array( $this, 'managePageLimitSaveAJAX' ) , FALSE );
			}
		}

		/**
		 * Add the page limit panel to the screen options of the manage page.
		 * NOTE: This relies on the the Screen Options class by Janis Elsts
		 */
		public function managePageLimit() {
			global $connections;

			$page = $connections->currentUser->getFilterPage( 'manage' );

			$out = '<label><input type="text" class="entry-per-page" name="wp_screen_options[value]" id="edit_entry_per_page" maxlength="3" value="' . $page->limit . '" />' . __( 'Entries', 'connections' ) . '</label>';
			$out .= '<input type="hidden" name="wp_screen_options[option]" id="edit_entry_per_page_name" value="connections" />';
			$out .= '<input type="submit" name="screen-options-apply" id="entry-per-page-apply" class="button" value="Apply"  />';
			return $out;
		}

		/**
		 * Save the user setting for the page limit on the screen options of the manage page.
		 * NOTE: This is only run during the AJAX callback which is currently disabled.
		 * NOTE: This relies on the the Screen Options class by Janis Elsts
		 */
		public function managePageLimitSaveAJAX() {
			include_once CN_PATH . '/includes/inc.processes.php';

			processSetUserFilter();
		}

		public function managePageLimitSave( $false = FALSE , $option , $value ) {
			global $connections;

			$user_meta = get_user_meta( $connections->currentUser->getID() , $option, TRUE );

			$user_meta['filter']['manage']['limit'] = absint( $value );
			$user_meta['filter']['manage']['current'] = 1;

			return $user_meta;
		}

		/**
		 * Register the admin menus for Connections
		 *
		 * @access private
		 * @since unknown
		 * @author Steven A. Zahm
		 * @since Undefined
		 * @return void
		 */
		public function adminMenu() {

			if ( ! current_user_can( 'connections_view_menu') ) return;

			// Set the capability string to be used in the add_sub_menu function per role capability assigned to the current user.
			if ( current_user_can( 'connections_add_entry_moderated' ) ) {
				$addEntryCapability = 'connections_add_entry_moderated';
			}
			elseif ( current_user_can( 'connections_add_entry' ) ) {
				$addEntryCapability = 'connections_add_entry';
			}
			else {
				$addEntryCapability = 'connections_add_entry_moderated';
			}

			// Register the top level menu item.
			$this->pageHook->topLevel = add_menu_page( 'Connections', 'Connections', 'connections_view_dashboard', 'connections_dashboard', array ( &$this, 'showPage' ), CN_URL . 'images/menu.png' );

			$submenu[0]   = array( 'hook' => 'dashboard', 'page_title' => 'Connections : ' . __( 'Dashboard', 'connections' ), 'menu_title' => __( 'Dashboard', 'connections' ), 'capability' => 'connections_view_dashboard', 'menu_slug' => 'connections_dashboard', 'function' => array ( &$this, 'showPage' ) );
			$submenu[20]  = array( 'hook' => 'manage', 'page_title' => 'Connections : ' . __( 'Manage', 'connections' ), 'menu_title' => __( 'Manage', 'connections' ), 'capability' => 'connections_manage', 'menu_slug' => 'connections_manage', 'function' => array ( &$this, 'showPage' ) );
			$submenu[40]  = array( 'hook' => 'add', 'page_title' => 'Connections : ' . __( 'Add Entry', 'connections' ), 'menu_title' => __( 'Add Entry', 'connections' ), 'capability' => $addEntryCapability, 'menu_slug' => 'connections_add', 'function' => array ( &$this, 'showPage' ) );
			$submenu[60]  = array( 'hook' => 'categories', 'page_title' => 'Connections : ' . __( 'Categories', 'connections' ), 'menu_title' => __( 'Categories', 'connections' ), 'capability' => 'connections_edit_categories', 'menu_slug' => 'connections_categories', 'function' => array ( &$this, 'showPage' ) );
			$submenu[80]  = array( 'hook' => 'templates', 'page_title' => 'Connections : ' . __( 'Templates', 'connections' ), 'menu_title' => __( 'Templates', 'connections' ), 'capability' => 'connections_manage_template', 'menu_slug' => 'connections_templates', 'function' => array ( &$this, 'showPage' ) );
			$submenu[100] = array( 'hook' => 'settings', 'page_title' => 'Connections : ' . __( 'Settings', 'connections' ), 'menu_title' => __( 'Settings', 'connections' ), 'capability' => 'connections_change_settings', 'menu_slug' => 'connections_settings', 'function' => array ( &$this, 'showPage' ) );
			$submenu[120] = array( 'hook' => 'roles', 'page_title' => 'Connections : ' . __( 'Roles &amp; Capabilites', 'connections' ), 'menu_title' => __( 'Roles', 'connections' ), 'capability' => 'connections_change_roles', 'menu_slug' => 'connections_roles', 'function' => array ( &$this, 'showPage' ) );
			//$submenu[140] = array( 'hook' => 'help', 'page_title' => 'Connections : ' . __('Help', 'connections'), 'menu_title' => __('Help', 'connections'), 'capability' => 'connections_view_help', 'menu_slug' => 'connections_help', 'function' => array (&$this, 'showPage') );

			$submenu = apply_filters( 'cn_submenu', $submenu );

			ksort( $submenu );

			foreach ( $submenu as $menu ) {
				extract( $menu );
				$this->pageHook->{$hook} = add_submenu_page( 'connections_dashboard', $page_title, $menu_title, $capability, $menu_slug, $function );
			}

		}

		/**
		 * Register the metaboxes used for editing an entry.
		 *
		 * Action added in connectionsLoad::adminInit
		 *
		 * @author Steven A. Zahm
		 * @since 0.7.1.3
		 * @return void
		 */
		public function registerEditMetaboxes() {
			// The meta boxes do not need diplayed/registered if no action is being taken on an entry. Such as copy/edit.
			if ( $_GET['page'] === 'connections_manage' &&  ! isset( $_GET['action'] ) )  return;

			$form = new cnFormObjects();

			$form->registerEditMetaboxes( substr( current_filter(), 5 ) );

			add_filter( 'screen_layout_columns', array( &$this, 'screenLayout' ), 10, 2 );
		}

		/**
		 * Register the metaboxes used for the Dashboard.
		 *
		 * Action added in connectionsLoad::adminInit
		 *
		 * @author Steven A. Zahm
		 * @since 0.7.1.6
		 * @return void
		 */
		public function registerDashboardMetaboxes() {
			$form = new cnFormObjects();
			$form->registerDashboardMetaboxes();

			add_filter( 'screen_layout_columns', array( &$this, 'screenLayout' ), 10, 2 );
		}

		/**
		 * Register the number of columns permitted for metabox use on the edit entry page.
		 *
		 * Filter added in connectionsLoad::registerEditMetaboxes
		 *
		 * @author Steven A. Zahm
		 * @since 0.7.1.3
		 * @return array
		 */
		public function screenLayout( $columns, $screen ) {
			$columns[$this->pageHook->dashboard] = 2;
			$columns[$this->pageHook->manage] = 2;
			$columns[$this->pageHook->add] = 2;

			return $columns;
		}

		/**
		 * Register the external JS libraries that may be enqueued in either the admin or frontend.
		 *
		 * @access private
		 * @since 0.7.3.2
		 * @uses wp_register_script()
		 * @return void
		 */
		public static function registerScripts() {
			global $connections;

			/*
			 * If the Google Maps API is disabled, do not register it and change the dependencies of
			 * both goMap and MarkerClusterer. Allowing the Google Maps API to be turned "off" provides
			 * compatibility with themes and other plugins the enqueue Google Maps but do not provide a
			 * method to disable it. So I will, unless we're in the admin, because the geocode function will
			 * require it.
			 */
			if ( $connections->options->getGoogleMapsAPI() || is_admin() ) {
				wp_register_script( 'cn-google-maps-api', 'http://maps.google.com/maps/api/js?sensor=false', array( 'jquery' ), CN_CURRENT_VERSION, $connections->options->getJavaScriptFooter() );
				wp_register_script( 'jquery-gomap-min', CN_URL . 'js/jquery.gomap-1.3.2.min.js', array( 'jquery' , 'cn-google-maps-api' ), '1.3.2', $connections->options->getJavaScriptFooter() );
				wp_register_script( 'jquery-markerclusterer-min', CN_URL . 'js/jquery.markerclusterer-2.0.10.min.js', array( 'jquery' , 'cn-google-maps-api' , 'jquery-gomap-min' ), '2.0.10', $connections->options->getJavaScriptFooter() );
			} else {
				wp_register_script( 'jquery-gomap-min', CN_URL . 'js/jquery.gomap-1.3.2.min.js', array( 'jquery' ), '1.3.2', $connections->options->getJavaScriptFooter() );
				wp_register_script( 'jquery-markerclusterer-min', CN_URL . 'js/jquery.markerclusterer.min.js', array( 'jquery' , 'jquery-gomap-min' ), '2.0.15', $connections->options->getJavaScriptFooter() );
			}

			if ( is_admin() ) {
				wp_register_script( 'cn-ui-admin', CN_URL . 'js/cn-admin.js', array( 'jquery' ), CN_CURRENT_VERSION, TRUE );
				wp_register_script( 'cn-widget', CN_URL . 'js/widgets.js', array( 'jquery' ), CN_CURRENT_VERSION, TRUE );
			} else {
				wp_register_script( 'cn-ui', CN_URL . 'js/cn-user.js', array( 'jquery', 'jquery-preloader' ), CN_CURRENT_VERSION, $connections->options->getJavaScriptFooter() );
			}

			wp_register_script( 'jquery-qtip', CN_URL . 'js/jquery.qtip.min.js', array( 'jquery' ), '2.0.1', $connections->options->getJavaScriptFooter() );
			wp_register_script( 'jquery-preloader', CN_URL . 'js/jquery.preloader.js', array( 'jquery' ), '1.1', $connections->options->getJavaScriptFooter() );

			// Disble this for now, Elegant Theme uses the same registration name in the admin which causes errors.
			// wp_register_script('jquery-spin', CN_URL . 'js/jquery.spin.js', array('jquery'), '1.2.5', $connections->options->getJavaScriptFooter() );

			wp_register_script( 'jquery-chosen-min', CN_URL . 'js/jquery.chosen.min.js', array( 'jquery' ), '0.9.11', $connections->options->getJavaScriptFooter() );
			wp_register_script( 'jquery-validate' , CN_URL . 'js/jquery.validate.min.js', array( 'jquery', 'jquery-form' ) , '1.9.0' , $connections->options->getJavaScriptFooter() );
		}

		/**
		 * Registers the CSS libraries that may be enqueued in the admin or frontend.
		 *
		 * @access private
		 * @since 0.7.3.2
		 * @uses wp_register_style()
		 * @return void
		 */
		public static function registerCSS() {

			if ( is_admin() ) {
				wp_register_style( 'cn-admin', CN_URL . 'css/cn-admin.css', array(), CN_CURRENT_VERSION );
				wp_register_style( 'cn-admin-jquery-ui', CN_URL . 'css/jquery-ui-' . ( 'classic' == get_user_option( 'admin_color' ) ? 'classic' : 'fresh' ) . '.css', array(), CN_CURRENT_VERSION );
			} else {
				wp_register_style( 'connections-user', CN_URL . 'css/cn-user.css', array(), CN_CURRENT_VERSION );
				wp_register_style( 'connections-qtip', CN_URL . 'css/jquery.qtip.min.css', array(), 'nightly' );
			}

			wp_register_style( 'connections-chosen', CN_URL . 'css/chosen.css', array(), '0.9.8' );
		}

		/**
		 * Enqueues the Connections JavaScript libraries on required admin pages.
		 *
		 * @access private
		 * @since 0.7.3.2
		 * @uses wp_enqueue_script()
		 * @param  string $pageHook The current admin page hook.
		 * @return void
		 */
		public static function enqueueAdminScripts( $pageHook ) {
			global $connections;

			if ( ! current_user_can( 'connections_view_menu') ) return;

			// Load on all the Connections admin pages.
			if ( in_array( $pageHook, get_object_vars( $connections->pageHook ) ) ) {
				wp_enqueue_script( 'cn-ui-admin' );
				wp_enqueue_script( 'jquery-preloader' );
			}

			/*
			 * TinyMCE in WordPress Plugins
			 * http://www.keighl.com/2010/01/tinymce-in-wordpress-plugins/
			 *
			 * For full editor see:
			 * http://dannyvankooten.com/450/tinymce-wysiwyg-editor-in-wordpress-plugin/
			 *
			 * Load the tinyMCE scripts on these pages.
			 */
			$editorPages = array( $connections->pageHook->manage, $connections->pageHook->add );

			if ( in_array( $pageHook, $editorPages ) ) {
				global $concatenate_scripts, $compress_scripts, $compress_css;

				wp_enqueue_script( 'jquery-gomap-min' );
				wp_enqueue_script( 'jquery-ui-datepicker' );
				wp_enqueue_script( 'jquery-chosen-min' );

				if ( version_compare( $GLOBALS['wp_version'], '3.2.999', '<' ) ) {
					$compress_scripts = FALSE; // If the script are compress the TinyMCE doesn't seem to function.

					wp_tiny_mce(
						FALSE , // true makes the editor "teeny"
						array(
							'editor_selector'         => 'tinymce',
							'theme_advanced_buttons1' => 'bold, italic, underline, |, bullist, numlist, |, justifyleft, justifycenter, justifyright, |, link, unlink, |, pastetext, pasteword, removeformat, |, undo, redo',
							'theme_advanced_buttons2' => '',
							'inline_styles'           => TRUE,
							'relative_urls'           => FALSE,
							'remove_linebreaks'       => FALSE,
							'plugins'                 => 'paste'
						)
					);
				}
			}

			// Load the core JavaScripts required for meta box UI.
			$metaBoxPages = array( $connections->pageHook->dashboard, $connections->pageHook->manage, $connections->pageHook->add );

			if ( in_array( $pageHook, $metaBoxPages ) ) {
				wp_enqueue_script( 'common' );
				wp_enqueue_script( 'wp-lists' );
				wp_enqueue_script( 'postbox' );
				wp_enqueue_script( 'cn-widget' );
			}
		}

		/**
		 * Enqueues the Connections JavaScript libraries on the frontend.
		 *
		 * @access private
		 * @since 0.7.3.2
		 * @uses wp_enqueue_script()
		 * @return void
		 */
		public static function enqueueScripts() {
			/*
			 * http://beerpla.net/2010/01/13/wordpress-plugin-development-how-to-include-css-and-javascript-conditionally-and-only-when-needed-by-the-posts/
			 * http://beerpla.net/2010/01/15/follow-up-to-loading-css-and-js-conditionally/
			 * http://scribu.net/wordpress/optimal-script-loading.html
			 */

			wp_enqueue_script( 'cn-ui' );
		}

		/**
		 * Enqueues the Connections CSS on the required admin pages.
		 *
		 * @access private
		 * @since 0.7.3.2
		 * @uses wp_enqueue_style()
		 * @param  string $pageHook The current admin page hook.
		 * @return void
		 */
		public static function enqueueAdminStyles( $pageHook ) {
			global $connections;

			if ( ! current_user_can( 'connections_view_menu') ) return;

			// Load on all the Connections admin pages.
			if ( in_array( $pageHook, get_object_vars( $connections->pageHook ) ) ) {
				wp_enqueue_style( 'cn-admin' );
				wp_enqueue_style( 'cn-admin-jquery-ui' );
			}

			// Load the WordPress widgets styles only on these pages.
			$adminPageEntryEdit = array( $connections->pageHook->manage, $connections->pageHook->add );

			if ( in_array( $pageHook, $adminPageEntryEdit ) ) {

				// Earlier version of WP had the widgets CSS in a seperate file.
				if ( version_compare( $GLOBALS['wp_version'], '3.2.999', '<' ) ) wp_enqueue_style( 'connections-admin-widgets', get_admin_url() . 'css/widgets.css' );

				wp_enqueue_style( 'connections-chosen' );
			}
		}

		/**
		 * Enqueues the Connections CSS on the frontend.
		 *
		 * @access private
		 * @since 0.7.3.2
		 * @uses wp_enqueue_style()
		 * @return void
		 */
		public static function enqueueStyles() {
			wp_enqueue_style( 'connections-user' );
			wp_enqueue_style( 'connections-chosen' );
			wp_enqueue_style( 'connections-qtip' );
		}

		// Add settings option
		public function addActionLinks( $links ) {
			$new_links = array();

			$new_links[] = '<a href="admin.php?page=connections_settings">' . __( 'Settings', 'connections' ) . '</a>';

			return array_merge( $new_links, $links );
		}

		// Add FAQ and support information
		public function addMetaLinks( $links, $file ) {
			if ( $file == CN_BASE_NAME ) {
				$links[] = '<a href="http://connections-pro.com/?page_id=29" target="_blank">' . __( 'Add-on Plugins', 'connections' ) . '</a>';
				$links[] = '<a href="http://connections-pro.com/?page_id=419" target="_blank">' . __( 'Templates', 'connections' ) . '</a>';
				$links[] = '<a href="http://connections-pro.com/documentation/plugin/" target="_blank">' . __( 'Documentation', 'connections' ) . '</a>';
				$links[] = '<a href="http://connections-pro.com/support" target="_blank">' . __( 'Support Forums', 'connections' ) . '</a>';
			}

			return $links;
		}

		/**
		 * Add the changelog as a table row on the Manage Plugin admin screen.
		 * Code based on Changelogger.
		 */
		public function displayUpgradeNotice() {
			include_once ABSPATH . 'wp-admin/includes/plugin-install.php';
			//echo "<tr><td colspan='5'>TEST</td></tr>";
			//$api = plugins_api('plugin_information', array('slug' => 'connections', 'fields' => array('tested' => true, 'requires' => false, 'rating' => false, 'downloaded' => false, 'downloadlink' => false, 'last_updated' => false, 'homepage' => false, 'tags' => false, 'sections' => true) ));
			//print_r($api);

			if ( version_compare( $GLOBALS['wp_version'], '2.9.999', '>' ) ) // returning bool if at least WP 3.0 is running
				$current = get_option( '_site_transient_update_plugins' );

			elseif ( version_compare( $GLOBALS['wp_version'], '2.7.999', '>' ) ) // returning bool if at least WP 2.8 is running
				$current = get_transient( 'update_plugins' );

			else
				$current = get_option( 'update_plugins' );

			//print_r($current);

			if ( !isset( $current->response[ CN_BASE_NAME ] ) ) return NULL;

			$r = $current->response[ CN_BASE_NAME ]; // response should contain the slug and upgrade_notice within an array.
			//print_r($r);

			if ( isset( $r->upgrade_notice ) ) {
				$columns = CLOSMINWP28 ? 3 : 5;

				$output .= '<tr class="plugin-update-tr"><td class="plugin-update" colspan="' . $columns . '"><div class="update-message" style="font-weight: normal;">';
				$output .= '<strong>Upgrade notice for version: ' . $r->new_version . '</strong>';
				$output .= '<ul style="list-style-type: square; margin-left:20px;"><li>' . $r->upgrade_notice . '</li></ul>';
				$output .= '</div></td></tr>';

				echo $output;
			}


			/*stdClass Object
			(
			    [id] => 5801
			    [slug] => connections
			    [new_version] => 0.7.0.0
			    [upgrade_notice] => Upgrading to this version might break custom templates.
			    [url] => http://wordpress.org/extend/plugins/connections/
			    [package] => http://downloads.wordpress.org/plugin/connections.0.7.0.0.zip
			)*/
		}

		/**
		 * This is the registered function calls for the Connections admin pages as registered
		 * using the add_menu_page() and add_submenu_page() WordPress functions.
		 *
		 * @since unknown
		 * @access private
		 * @return void
		 */
		public function showPage() {

			if ( $this->dbUpgrade ) {
				include_once dirname( __FILE__ ) . '/includes/inc.upgrade.php';
				connectionsShowUpgradePage();
				return;
			}

			switch ( $_GET['page'] ) {
				case 'connections_dashboard':
					include_once CN_PATH . '/submenus/dashboard.php';
					connectionsShowDashboardPage();
					break;

				case 'connections_manage':
					include_once CN_PATH . '/submenus/manage.php';
					( isset( $_GET['action'] ) && ! empty( $_GET['action'] ) ) ? $action = $_GET['action'] : $action = '';

					connectionsShowViewPage( $action );
					break;

				case 'connections_add':
					include_once CN_PATH . '/submenus/manage.php';

					connectionsShowViewPage( 'add' );
					break;

				case 'connections_categories':
					include_once CN_PATH . '/submenus/categories.php';
					connectionsShowCategoriesPage();
					break;

				case 'connections_settings':
					include_once CN_PATH . '/submenus/settings.php';
					connectionsShowSettingsPage();
					break;

				case 'connections_templates':
					include_once CN_PATH . '/submenus/templates.php';
					connectionsShowTemplatesPage();
					break;

				case 'connections_roles':
					include_once CN_PATH . '/submenus/roles.php';
					connectionsShowRolesPage();
					break;

				case 'connections_help':
					include_once CN_PATH . '/submenus/help.php';
					connectionsShowHelpPage();
					break;
			}

		}

		/**
		 * Verify and process requested actions in the admin.
		 *
		 * @since unknown
		 * @access private
		 * @return void
		 */
		public function adminActions( $wp ) {
			// Exit the method if $_GET['connections_process'] isn't set.
			if ( ! isset( $_GET['connections_process'] ) ) return;

			global $connections;

			include_once CN_PATH . '/includes/inc.processes.php';
			$form = new cnFormObjects();

			switch ( $_GET['process'] ) {
				case 'manage':
					if ( isset( $_GET['action'] ) && ! empty( $_GET['action'] ) ) {
						switch ( $_GET['action'] ) {
							case 'add':
								/*
								 * Check whether the current user can add an entry.
								 */
								if ( current_user_can( 'connections_add_entry' ) || current_user_can( 'connections_add_entry_moderated' ) ) {
									check_admin_referer( $form->getNonce( 'add_entry' ), '_cn_wpnonce' );

									// Setup the redirect URL.
									$redirect = isset( $_POST['redirect'] ) ? $_POST['redirect'] : 'admin.php?page=connections_add';

									processEntry( $_POST, 'add' );

									wp_redirect( get_admin_url( get_current_blog_id(), $redirect) );
									exit();
								} else {
									$connections->setErrorMessage( 'capability_add' );
								}
								break;

							case 'update':
								/*
								 * Check whether the current user can edit an entry.
								 */
								if ( current_user_can( 'connections_edit_entry' ) || current_user_can( 'connections_edit_entry_moderated' ) ) {
									check_admin_referer( $form->getNonce( 'update_entry' ), '_cn_wpnonce' );

									// Setup the redirect URL.
									$redirect = isset( $_POST['redirect'] ) ? $_POST['redirect'] : 'admin.php?page=connections_manage';

									processEntry( $_POST, 'update' );

									wp_redirect( get_admin_url( get_current_blog_id(), $redirect ) );
									exit();
								} else {
									$connections->setErrorMessage( 'capability_edit' );
								}
								break;

							case 'approve':
								/*
								 * Check whether the current user can edit an entry.
								 */
								if ( current_user_can( 'connections_edit_entry' ) ) {
									processSetEntryStatus( 'approved' );
									wp_redirect( get_admin_url( get_current_blog_id(), 'admin.php?page=connections_manage' ) );
									exit();
								} else {
									$connections->setErrorMessage( 'capability_edit' );
								}
								break;

							case 'unapprove':
								/*
								 * Check whether the current user can edit an entry.
								 */
								if ( current_user_can( 'connections_edit_entry' ) ) {
									processSetEntryStatus( 'pending' );
									wp_redirect( get_admin_url( get_current_blog_id(), 'admin.php?page=connections_manage' ) );
									exit();
								} else {
									$connections->setErrorMessage( 'capability_edit' );
								}
								break;

							case 'delete':
								/*
								 * Check whether the current user delete an entry.
								 */
								if ( current_user_can( 'connections_delete_entry' ) ) {
									processDeleteEntry();
									wp_redirect( get_admin_url( get_current_blog_id(), 'admin.php?page=connections_manage' ) );
									exit();
								} else {
									$connections->setErrorMessage( 'capability_delete' );
								}
								break;

							case 'filter':
								check_admin_referer( 'filter' );
								processSetUserFilter();
								wp_redirect( get_admin_url( get_current_blog_id(), add_query_arg( array( 's' => (  isset( $_POST['s'] ) ? urlencode( $_POST['s'] ) : '' ) ) , 'admin.php?page=connections_manage' ) ) );
								exit();
								break;

							case 'do':
								if ( isset( $_POST['action'] ) && ! empty( $_POST['action'] ) ) {
									switch ( $_POST['action'] ) {
										case 'delete':
											/*
											 * Check whether the current user delete an entry.
											 */
											if ( current_user_can( 'connections_delete_entry' ) ) {
												check_admin_referer( $form->getNonce( 'bulk_action' ), '_cn_wpnonce' );
												processDeleteEntries();
												wp_redirect( get_admin_url( get_current_blog_id(), add_query_arg( array( 's' => urlencode( $_POST['s'] ) ) , 'admin.php?page=connections_manage' ) ) );
												exit();
											} else {
												$connections->setErrorMessage( 'capability_delete' );
											}
											break;

										case 'approve':
											/*
											 * Check whether the current user delete an entry.
											 */
											if ( current_user_can( 'connections_edit_entry' ) ) {
												check_admin_referer( $form->getNonce( 'bulk_action' ), '_cn_wpnonce' );
												processSetEntryStatuses( 'approved' );
												wp_redirect( get_admin_url( get_current_blog_id(), add_query_arg( array( 's' => urlencode( $_POST['s'] ) ) , 'admin.php?page=connections_manage' ) ) );
												exit();
											} else {
												$connections->setErrorMessage( 'capability_edit' );
											}
											break;

										case 'unapprove':
											/*
											 * Check whether the current user delete an entry.
											 */
											if ( current_user_can( 'connections_edit_entry' ) ) {
												check_admin_referer( $form->getNonce( 'bulk_action' ), '_cn_wpnonce' );
												processSetEntryStatuses( 'pending' );
												wp_redirect( get_admin_url( get_current_blog_id(), add_query_arg( array( 's' => urlencode( $_POST['s'] ) ) , 'admin.php?page=connections_manage' ) ) );
												exit();
											} else {
												$connections->setErrorMessage( 'capability_edit' );
											}
											break;

										case 'public' || 'private' || 'unlisted':
											/*
											 * Check whether the current user can edit entries.
											 */
											if ( current_user_can( 'connections_edit_entry' ) ) {
												check_admin_referer( $form->getNonce( 'bulk_action' ), '_cn_wpnonce' );
												processSetEntryVisibility();
												wp_redirect( get_admin_url( get_current_blog_id(), add_query_arg( array( 's' => urlencode( $_POST['s'] ) ) , 'admin.php?page=connections_manage' ) ) );
												exit();
											} else {
												$connections->setErrorMessage( 'capability_edit' );
											}
											break;

										default:
											wp_redirect( get_admin_url( get_current_blog_id(), add_query_arg( array( 's' => urlencode( $_POST['s'] ) ) , 'admin.php?page=connections_manage' ) ) );
											exit();
											break;
									}
								}

								check_admin_referer( $form->getNonce( 'bulk_action' ), '_cn_wpnonce' );
								processSetUserFilter();
								wp_redirect( get_admin_url( get_current_blog_id(), add_query_arg( array( 's' => urlencode( $_POST['s'] ) ) , 'admin.php?page=connections_manage' ) ) );
								exit();
								break;
						}
					}

					break;

				case 'category':
					/*
					 * Check whether user can edit Settings
					 */
					if ( current_user_can( 'connections_edit_categories' ) ) {
						if ( $_GET['action'] ) {
							switch ( $_GET['action'] ) {
								case 'add':
									check_admin_referer( $form->getNonce( 'add_category' ), '_cn_wpnonce' );
									processAddCategory();
									wp_redirect( get_admin_url( get_current_blog_id(), 'admin.php?page=connections_categories' ) );
									exit();
									break;

								case 'update':
									check_admin_referer( $form->getNonce( 'update_category' ), '_cn_wpnonce' );
									processUpdateCategory();
									wp_redirect( get_admin_url( get_current_blog_id(), 'admin.php?page=connections_categories' ) );
									exit();
									break;

								case 'delete':
									processDeleteCategory( 'delete' );
									wp_redirect( get_admin_url( get_current_blog_id(), 'admin.php?page=connections_categories' ) );
									exit();
									break;

								case 'bulk_delete':
									check_admin_referer( $form->getNonce( 'bulk_delete_category' ), '_cn_wpnonce' );
									processDeleteCategory( 'bulk_delete' );
									wp_redirect( get_admin_url( get_current_blog_id(), 'admin.php?page=connections_categories' ) );
									exit();
									break;
							}
						}
					}
					else {
						$connections->setErrorMessage( 'capability_categories' );
					}
					break;

				case 'template':
					/*
					 * Check whether user can manage Templates
					 */
					if ( current_user_can( 'connections_manage_template' ) ) {
						if ( $_GET['action'] ) {
							switch ( $_GET['action'] ) {
								case 'activate':
									processActivateTemplate();

									( ! isset( $_GET['type'] ) ) ? $tab = 'all' : $tab = esc_attr( $_GET['type'] );
									wp_redirect( get_admin_url( get_current_blog_id(), add_query_arg( array( 'type' => $tab ) , 'admin.php?page=connections_templates' ) ) );
									exit();
									break;

								case 'install':
									check_admin_referer( $form->getNonce( 'install_template' ), '_cn_wpnonce' );
									processInstallTemplate();

									( ! isset( $_GET['type'] ) ) ? $tab = 'all' : $tab = esc_attr( $_GET['type'] );
									wp_redirect( get_admin_url( get_current_blog_id(), add_query_arg( array( 'type' => $tab ) , 'admin.php?page=connections_templates' ) ) );
									exit();
									break;

								case 'delete':
									processDeleteTemplate();

									( ! isset( $_GET['type'] ) ) ? $tab = 'all' : $tab = esc_attr( $_GET['type'] );
									wp_redirect( get_admin_url( get_current_blog_id(), add_query_arg( array( 'type' => $tab ) , 'admin.php?page=connections_templates' ) ) );
									exit();
									break;
							}
						}
					} else {
						// @TODO: Create template specific error message.
						$connections->setErrorMessage( 'capability_settings' );
					}

					break;

				case 'role':

					/*
					 * Check whether user can edit roles
					 */
					if ( current_user_can( 'connections_change_roles' ) ) {
						if ( $_GET['action'] === 'update' ) {
							check_admin_referer( $form->getNonce( 'update_role_settings' ), '_cn_wpnonce' );
							updateRoleSettings();
							wp_redirect( get_admin_url( get_current_blog_id(), 'admin.php?page=connections_roles' ) );
							exit();
						}
					} else {
						$connections->setErrorMessage( 'capability_roles' );
					}

					break;
			}
		}

		/**
		 * This action will handle frontend process requests, currently only creating the vCard for download.
		 *
		 * @TODO If no vcard is found should redirect to an error message.
		 * @access private
		 * @since 0.7.3
		 * @return void
		 */
		public function frontendActions() {
			global $connections;
			$process = get_query_var( 'cn-process' );
			$token = get_query_var( 'cn-token' );
			$id = (integer) get_query_var( 'cn-id' );

			if ( $process === 'vcard' ) {
				$slug = get_query_var( 'cn-entry-slug' ); //var_dump($slug);

				global $connections;

				/*
				 * If the token and id values were set, the link was likely from the admin.
				 * Check for those values and validate the token. The primary reason for this
				 * to be able to download vCards of entries that are set to "Unlisted".
				 */
				if ( ! empty( $id ) && ! empty( $token ) ) {
					if ( ! wp_verify_nonce( $token, 'download_vcard_' . $id ) ) wp_die( 'Invalid vCard Token' );

					$entry = $connections->retrieve->entry( $id );

					// Die if no entry was found.
					if ( empty( $entry ) ) wp_die( __( 'vCard not available for download.', 'connections' ) );

					$vCard = new cnvCard( $entry ); //var_dump($vCard);die;
				}
				else {
					$entry = $connections->retrieve->entries( array( 'slug' => $slug ) ); //var_dump($entry);die;

					// Die if no entry was found.
					if ( empty( $entry ) ) wp_die( __( 'vCard not available for download.', 'connections' ) );

					$vCard = new cnvCard( $entry[0] ); //var_dump($vCard);die;
				}


				$filename = sanitize_file_name( $vCard->getName() ); //var_dump($filename);
				$data = $vCard->getvCard(); //var_dump($data);die;


				header( 'Content-Type: text/x-vcard; charset=utf-8' );
				header( 'Content-Disposition: attachment; filename=' . $filename . '.vcf' );
				header( 'Content-Length: ' . strlen( $data ) );
				header( 'Pragma: public' );
				header( "Pragma: no-cache" );
				header( "Expires: 0" );
				header( 'Connection: close' );

				echo $data;
				exit;
			}
		}

		/**
		 *
		 *
		 * @TODO remove after 8/2013
		 * @access private
		 * @since unknown
		 * @deprecated since 0.7.3
		 * @param object  $wp
		 * @return
		 */
		public function userActions( $wp ) {
			//print_r('<pre style="background-color: white;">'); print_r($wp->query_vars); print_r('</pre>');

			/*if ( array_key_exists('cn-vc', $wp->query_vars) && $wp->query_vars['cnvc'] == '1' )
			{
				$token = esc_attr($_GET['cn-token']);
				$id = (integer) esc_attr($_GET['cn-id']);


				if ( ! wp_verify_nonce( $token, 'download_vcard_' . $id) ) wp_die('Invalid vCard Token');

				global $connections;

				$entry = $connections->retrieve->entry($id);
				$vCard = new cnvCard($entry);

				$filename = sanitize_file_name($vCard->getFullFirstLastName());
				$data = $vCard->getvCard();

				header('Content-Type: text/x-vcard; charset=utf-8');
				header('Content-Disposition: attachment; filename=' . $filename . '.vcf');
				header('Content-Length: ' . strlen($data) );
				header('Pragma: public');
				header("Pragma: no-cache");
				header("Expires: 0");
				header('Connection: close');

				echo $data;
				exit;
			}*/

			if ( array_key_exists( 'cnpagename', $wp->query_vars ) && $wp->query_vars['cnpagename'] == 'directory' ) {
				global $template, $wp_query;

				/*
				 * Don't want WordPress applying these filters. Messes up the template with the auto P and texturize.
				 */
				remove_filter( 'the_content', 'wpautop' );
				remove_filter( 'the_content', 'wptexturize' );

				/*
				 * Clear the query just incase there were some custom queryies being called elsewhere.
				 */
				wp_reset_query();
				wp_reset_postdata();

				$atts = array();

				//if ( array_key_exists('cnlisttype', $wp->query_vars) && $wp->query_vars['cnlisttype'] != '' ) $atts['list_type'] = $wp->query_vars['cnlisttype'];
				//if ( array_key_exists('cnid', $wp->query_vars) && $wp->query_vars['cnid'] != '' ) $atts['id'] = $wp->query_vars['cnid'];

				/*
				 * Picked up this section from http://wordpress.org/extend/plugins/virtual-pages/
				 * Learned a bit from http://www.binarymoon.co.uk/2010/02/creating-wordpress-permalink-structure-custom-content/
				 *
				 *
				 * START
				 */

				// Pagination -- Will definately need this later.
				/*if(array_key_exists('paged', $wp->query_vars)) {
				    $this->paged = $wp->query_vars['paged'];
				}
				else {
				    $this->paged = 1;
				}*/

				/*
				 * Trick wp_query into thinking this is a page (necessary for wp_title() at least)
				 * Not sure if it's cheating or not to modify global variables in a filter
				 * but it appears to work and the codex doesn't directly say not to.
				 */
				$wp_query->is_page = TRUE;
				$wp_query->is_singular = TRUE;
				$wp_query->is_home = FALSE;
				$wp_query->is_archive = FALSE;
				$wp_query->is_category = FALSE;

				//Longer permalink structures may not match the fake post slug and cause a 404 error so we catch the error here
				unset( $wp_query->query['error'] );
				$wp_query->query_vars['error'] = '';
				$wp_query->is_404 = FALSE;

				$wp_query->post_count = 1;

				/*
				 * Fake post ID to prevent WP from trying to show comments or
				 * showing the edit link for a post that doesn't really exist.
				 *
				 * Also used when creating the permalink. If ID >= 0 the filter
				 * that creates the permalink is not applied.
				 */
				$wp_query->post->ID = -1;

				/*
				 * The author ID for the post.  Usually 1 is the sys admin.
				 * @TODO Add option to the settings page to allow the defining of the page author.
				 */
				$wp_query->post->post_author = 94;

				/*
				 * When virtual pages are created in the admin, save the date created to be applied here.
				 */
				$wp_query->post->post_date = current_time( 'mysql' );
				$wp_query->post->post_date_gmt = current_time( 'mysql', 1 );

				$wp_query->post->post_content = '';

				/*
				 * Define the page title.
				 * @TODO Add option to the settings page to the page title.
				 */
				$wp_query->post->post_title = 'Connections Test';

				$wp_query->post->post_category = 0;

				$wp_query->post->post_excerpt = '';

				$wp_query->post->post_status = 'publish';

				$wp_query->post->comment_status = 'closed';

				$wp_query->post->ping_status = 'closed';

				$wp_query->post->post_password = '';

				/*
				 * This would be the page slug generated from the post title and used in the permalink.
				 * @TODO Well gotta set this.
				 */
				$wp_query->post->post_name = '';

				$wp_query->post->to_ping = '';

				$wp_query->post->pinged = '';

				/*
				 * When virtual pages are modified in the admin, save the date modfied to be applied here.
				 */
				$wp_query->post->post_modified = current_time( 'mysql' );
				$wp_query->post->post_modified_gmt = current_time( 'mysql', 1 );

				$wp_query->post->post_content_filtered = '';

				$wp_query->post->parent_post = 0;

				$wp_query->post->guid = esc_url( "http://".$_SERVER['HTTP_HOST'].$_SERVER['REQUEST_URI'] );

				$wp_query->post->menu_order = 0;

				$wp_query->post->post_type = 'page';

				$wp_query->post->post_mime_type = '';

				$wp_query->post->comment_count = 0;

				$wp_query->post->restricted = 0;

				$wp_query->post->ancestors = array();

				$wp_query->post->filter = 'raw';


				// Dupe the post data the posts array;
				$wp_query->posts[] =& $wp_query->post;

				/*
				 * Picked up this section from http://wordpress.org/extend/plugins/virtual-pages/
				 *
				 * END
				 */

				/*
				 * Setup the permalink.
				 */
				add_filter( 'page_link', array( &$this, 'createPermalink' ), 10, 3 );

				/*
				 * Set the attributes and then populate the content.
				 */
				add_filter( 'cn_list_template_init', array( &$this, 'setconnectionsListAttsPre' ) );
				add_filter( 'cn_list_atts', array( &$this, 'setconnectionsListAttsPre' ) );
				add_action( 'the_content', '_connections_list' );

				/*
				 * Grab the path to the theme's page template.
				 * @TODO Add option to the settings page to allow the selection of custom page templates.
				 */
				$template = get_query_template( 'page' );

				// Include the page template and if the $wp_query object was loaded correctly the page should 'just work'.
				include $template;

				// Exit so WP doesn't continue to try to process the page.
				exit;
			}

		}

		public function setconnectionsListAttsPre( $atts ) {
			global $wp;

			if ( array_key_exists( 'cnlt', $wp->query_vars ) && $wp->query_vars['cnlt'] != '' ) $atts['list_type'] = $wp->query_vars['cnlt'];
			if ( array_key_exists( 'cncatnm', $wp->query_vars ) && $wp->query_vars['cncatnm'] != '' ) $atts['category_slug'] = $wp->query_vars['cncatnm'];
			if ( array_key_exists( 'cnid', $wp->query_vars ) && $wp->query_vars['cnid'] != '' ) $atts['id'] = $wp->query_vars['cnid'];
			if ( array_key_exists( 'cnname', $wp->query_vars ) && $wp->query_vars['cnname'] != '' ) $atts['slug'] = $wp->query_vars['cnname'];

			return $atts;
		}

		public function createPermalink( $link = '', $id = 0, $sample = FALSE ) {
			if ( $id >= 0 ) {
				return $link;
			}
			else {
				return esc_url( 'http://' . $_SERVER['HTTP_HOST'] . $_SERVER['REDIRECT_URL'] );
			}
		}
	}

	global $connections;
	$connections = new connectionsLoad();

}